@@ -37,8 +37,8 @@
 #ifndef LIBTORRENT_LISTEN_H
 #define LIBTORRENT_LISTEN_H
 
-#include <inttypes.h>
-#include lt_tr1_functional
+#include <cinttypes>
+#include <functional>
 #include <rak/socket_address.h>
 
 #include "socket_base.h"
@@ -48,10 +48,9 @@ namespace torrent {
 
 class Listen : public SocketBase {
 public:
-  typedef std::function<void (SocketFd, const rak::socket_address&)> slot_connection;
+  using slot_connection = std::function<void(SocketFd, const rak::socket_address&)>;
 
-  Listen() : m_port(0) {}
-  ~Listen() { close(); }
+  ~Listen() override { close(); }
 
   bool                open(uint16_t first, uint16_t last, int backlog, const rak::socket_address* bindAddress);
   void                close();
@@ -62,12 +61,12 @@ public:
 
   slot_connection&    slot_accepted() { return m_slot_accepted; }
 
-  virtual void        event_read();
-  virtual void        event_write();
-  virtual void        event_error();
+  void                event_read() override;
+  void                event_write() override;
+  void                event_error() override;
 
 private:
-  uint64_t            m_port;
+  uint64_t            m_port{0};
 
   slot_connection     m_slot_accepted;
 };
