@@ -1,64 +1,25 @@
-// libTorrent - BitTorrent library
-// Copyright (C) 2005-2011, Jari Sundell
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-// In addition, as a special exception, the copyright holders give
-// permission to link the code of portions of this program with the
-// OpenSSL library under certain conditions as described in each
-// individual source file, and distribute linked combinations
-// including the two.
-//
-// You must obey the GNU General Public License in all respects for
-// all of the code used other than OpenSSL.  If you modify file(s)
-// with this exception, you may extend this exception to your version
-// of the file(s), but you are not obligated to do so.  If you do not
-// wish to do so, delete this exception statement from your version.
-// If you delete this exception statement from all source files in the
-// program, then also delete it here.
-//
-// Contact:  Jari Sundell <jaris@ifi.uio.no>
-//
-//           Skomakerveien 33
-//           3185 Skoppum, NORWAY
-
 #include "config.h"
 
+#include "resume.h"
+
 #include <rak/file_stat.h>
 #include <rak/socket_address.h>
 
-#include "peer/peer_info.h"
-#include "peer/peer_list.h"
-#include "torrent/utils/log.h"
-
 #include "data/file.h"
 #include "data/file_list.h"
 #include "data/transfer_list.h"
+#include "download/download_main.h"
 #include "net/address_list.h"
-
-#include "common.h"
-#include "bitfield.h"
-#include "download.h"
-#include "download_info.h"
-#include "object.h"
-#include "tracker.h"
-#include "tracker_list.h"
-
-#include "globals.h"
-
-#include "resume.h"
+#include "peer/peer_info.h"
+#include "peer/peer_list.h"
+#include "torrent/common.h"
+#include "torrent/bitfield.h"
+#include "torrent/download.h"
+#include "torrent/download_info.h"
+#include "torrent/object.h"
+#include "torrent/tracker/tracker.h"
+#include "torrent/utils/log.h"
+#include "tracker/tracker_list.h"
 
 #define LT_LOG_LOAD(log_fmt, ...)                                       \
   lt_log_print_info(LOG_RESUME_DATA, download.info(), "resume_load", log_fmt, __VA_ARGS__);
@@ -92,12 +53,14 @@ resume_load_progress(Download download, const Object& object) {
   if (!resume_load_bitfield(download, object))
     return;
 
-  Object::list_const_iterator filesItr  = files.begin();
+  auto filesItr  = files.begin();
 
   FileList* fileList = download.file_list();
 
-  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
-    std::string file_path = (*listItr)->path()->as_string();
+  for (auto listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
+    if ( (*listItr)->is_padding())
+      continue;
+
     unsigned int file_index = std::distance(fileList->begin(), listItr);
 
     rak::file_stat fs;
@@ -121,7 +84,7 @@ resume_load_progress(Download download, const Object& object) {
     // resize the file.
     (*listItr)->unset_flags(File::flag_create_queued | File::flag_resize_queued);
 
-    if (mtimeValue == ~int64_t(0) || mtimeValue == ~int64_t(1)) {
+    if (mtimeValue == ~int64_t{0} || mtimeValue == ~int64_t{1}) {
       // If 'mtime' is ~0 it means we haven't gotten around to
       // creating the file.
       //
@@ -132,7 +95,7 @@ resume_load_progress(Download download, const Object& object) {
       // creating the file. It will just fail on the mtime check
       // later, so we don't need to handle it explicitly.
 
-      if (mtimeValue == ~int64_t(0)) {
+      if (mtimeValue == ~int64_t{0}) {
         LT_LOG_LOAD_FILE("file not created by client, file:create|resize range:clear|(recheck)", 0);
         (*listItr)->set_flags(File::flag_create_queued | File::flag_resize_queued);
       } else {
@@ -148,7 +111,7 @@ resume_load_progress(Download download, const Object& object) {
 
     // If the file is the wrong size, queue resize and clear resume
     // data for that file.
-    if ((uint64_t)fs.size() != (*listItr)->size_bytes()) {
+    if (static_cast<uint64_t>(fs.size()) != (*listItr)->size_bytes()) {
       if (fs.size() == 0) {
         LT_LOG_LOAD_FILE("zero-length file found, file:resize range:clear|recheck", 0);
       } else {
@@ -166,7 +129,7 @@ resume_load_progress(Download download, const Object& object) {
     // chunks that might not have been completely written to disk.
     //
     // This gets handled below, so just skip to the next file.
-    if (mtimeValue == ~int64_t(3)) {
+    if (mtimeValue == ~int64_t{3}) {
       LT_LOG_LOAD_FILE("file was downloading", 0);
       continue;
     }
@@ -179,7 +142,7 @@ resume_load_progress(Download download, const Object& object) {
     // the file, else clear the range. This should be set only for
     // files that have completed and got no indices in
     // TransferList::completed_list().
-    if (mtimeValue == ~int64_t(2) || mtimeValue != fs.modified_time()) {
+    if (mtimeValue == ~int64_t{2} || mtimeValue != fs.modified_time()) {
       LT_LOG_LOAD_FILE("resume data doesn't include uncertain pieces, range:clear|recheck", 0);
       download.update_range(Download::update_range_clear | Download::update_range_recheck,
                             (*listItr)->range().first, (*listItr)->range().second);
@@ -212,20 +175,20 @@ resume_save_progress(Download download, Object& object) {
 
     Object::list_type& files = object.get_key_list("files");
 
-    for (Object::list_iterator itr = files.begin(), last = files.end(); itr != last; itr++)
-      itr->insert_key("mtime", ~int64_t(2));
+    for (auto& file : files)
+      file.insert_key("mtime", ~int64_t{2});
 
     return;
   }
 
   resume_save_bitfield(download, object);
-  
+
   Object::list_type&    files    = object.insert_preserve_copy("files", Object::create_list()).first->second.as_list();
-  Object::list_iterator filesItr = files.begin();
+  auto filesItr = files.begin();
 
   FileList* fileList = download.file_list();
 
-  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
+  for (auto listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     unsigned int file_index = std::distance(fileList->begin(), listItr);
 
     if (filesItr == files.end())
@@ -233,20 +196,20 @@ resume_save_progress(Download download, Object& object) {
     else if (!filesItr->is_map())
       *filesItr = Object::create_map();
 
-    filesItr->insert_key("completed", (int64_t)(*listItr)->completed_chunks());
+    filesItr->insert_key("completed", static_cast<int64_t>((*listItr)->completed_chunks()));
 
     rak::file_stat fs;
     bool fileExists = fs.update(fileList->root_dir() + (*listItr)->path()->as_string());
 
     if (!fileExists) {
-      
+
       if ((*listItr)->is_create_queued()) {
         // ~0 means the file still needs to be created.
-        filesItr->insert_key("mtime", ~int64_t(0));
+        filesItr->insert_key("mtime", ~int64_t{0});
         LT_LOG_SAVE_FILE("file not created, create queued", 0);
       } else {
         // ~1 means the file shouldn't be created.
-        filesItr->insert_key("mtime", ~int64_t(1));
+        filesItr->insert_key("mtime", ~int64_t{1});
         LT_LOG_SAVE_FILE("file not created, create not queued", 0);
       }
 
@@ -258,27 +221,27 @@ resume_save_progress(Download download, Object& object) {
 
       // This assumes the syncs are properly called before
       // resume_save_progress gets called after finishing a torrent.
-      filesItr->insert_key("mtime", (int64_t)fs.modified_time());
+      filesItr->insert_key("mtime", static_cast<int64_t>(fs.modified_time()));
       LT_LOG_SAVE_FILE("file completed, mtime:%" PRIi64, (int64_t)fs.modified_time());
 
     } else if (!download.info()->is_active()) {
       // When stopped, all chunks should have received sync, thus the
       // file's mtime will be correct. (We hope)
-      filesItr->insert_key("mtime", (int64_t)fs.modified_time());
+      filesItr->insert_key("mtime", static_cast<int64_t>(fs.modified_time()));
       LT_LOG_SAVE_FILE("file inactive and assumed sync'ed, mtime:%" PRIi64, (int64_t)fs.modified_time());
 
     } else {
       // If the torrent isn't done and we've not shut down, then set
       // 'mtime' to ~3 so as to indicate that the 'mtime' is not to be
       // trusted, yet we have a partial bitfield for the file.
-      filesItr->insert_key("mtime", ~int64_t(3));
+      filesItr->insert_key("mtime", ~int64_t{3});
       LT_LOG_SAVE_FILE("file actively downloading", 0);
     }
   }
 }
 
 void
-resume_clear_progress(Download download, Object& object) {
+resume_clear_progress([[maybe_unused]] Download download, Object& object) {
   object.erase_key("bitfield");
 }
 
@@ -294,7 +257,7 @@ resume_load_bitfield(Download download, const Object& object) {
 
     LT_LOG_LOAD("restoring partial bitfield", 0);
 
-    download.set_bitfield((uint8_t*)bitfield.c_str(), (uint8_t*)(bitfield.c_str() + bitfield.size()));
+    download.set_bitfield((uint8_t*)(bitfield.c_str()), (uint8_t*)((bitfield.c_str() + bitfield.size())));
 
   } else if (object.has_key_value("bitfield")) {
     Object::value_type chunksDone = object.get_key_value("bitfield");
@@ -327,7 +290,7 @@ resume_save_bitfield(Download download, Object& object) {
     object.insert_key("bitfield", bitfield->size_set());
   } else {
     LT_LOG_SAVE("saving bitfield", 0);
-    object.insert_key("bitfield", std::string((char*)bitfield->begin(), bitfield->size_bytes()));
+    object.insert_key("bitfield", std::string(bitfield->begin(), bitfield->end()));
   }
 }
 
@@ -339,8 +302,8 @@ resume_load_uncertain_pieces(Download download, const Object& object) {
     return;
   }
 
-  if(!object.has_key_value("uncertain_pieces.timestamp") ||
-     object.get_key_value("uncertain_pieces.timestamp") >= (int64_t)download.info()->load_date()) {
+  if (!object.has_key_value("uncertain_pieces.timestamp") ||
+      object.get_key_value("uncertain_pieces.timestamp") >= static_cast<int64_t>(download.info()->load_date())) {
     LT_LOG_LOAD_INVALID("invalid information on uncertain pieces", 0);
     return;
   }
@@ -349,11 +312,16 @@ resume_load_uncertain_pieces(Download download, const Object& object) {
 
   LT_LOG_LOAD("found %zu uncertain pieces", uncertain.size() / 2);
 
-  for (const char* itr = uncertain.c_str(), *last = uncertain.c_str() + uncertain.size();
-       itr + sizeof(uint32_t) <= last; itr += sizeof(uint32_t)) {
+  const char* itr  = uncertain.c_str();
+  const char* last = uncertain.c_str() + uncertain.size();
+
+  while (itr + sizeof(uint32_t) <= last) {
     // Fix this so it does full ranges.
     download.update_range(Download::update_range_recheck | Download::update_range_clear,
-                          ntohl(*(uint32_t*)itr), ntohl(*(uint32_t*)itr) + 1);
+                          ntohl(*reinterpret_cast<const uint32_t*>(itr)),
+                          ntohl(*reinterpret_cast<const uint32_t*>(itr)) + 1);
+
+    itr += sizeof(uint32_t);
   }
 }
 
@@ -365,9 +333,10 @@ resume_save_uncertain_pieces(Download download, Object& object) {
   object.erase_key("uncertain_pieces.timestamp");
 
   const TransferList::completed_list_type& completedList = download.transfer_list()->completed_list();
+
   auto itr = std::find_if(completedList.begin(), completedList.end(), [](const auto& v) {
-    return (rak::timer::current() - rak::timer::from_minutes(15).usec()) <= v.first;
-  });
+      return this_thread::cached_time() - 15min <= std::chrono::microseconds(v.first);
+    });
 
   if (itr == completedList.end())
     return;
@@ -380,16 +349,17 @@ resume_save_uncertain_pieces(Download download, Object& object) {
 
   std::sort(buffer.begin(), buffer.end());
 
-  for (std::vector<uint32_t>::iterator itr2 = buffer.begin(), last = buffer.end(); itr2 != last; itr2++)
-    *itr2 = htonl(*itr2);
+  for (unsigned int& itr2 : buffer)
+    itr2 = htonl(itr2);
+
+  object.insert_key("uncertain_pieces.timestamp", this_thread::cached_seconds().count());
 
-  object.insert_key("uncertain_pieces.timestamp", rak::timer::current_seconds());
   Object::string_type& completed = object.insert_key("uncertain_pieces", std::string()).as_string();
-  completed.append((const char*)&buffer.front(), buffer.size() * sizeof(uint32_t));
+  completed.append(reinterpret_cast<const char*>(&buffer.front()), buffer.size() * sizeof(uint32_t));
 }
 
 bool
-resume_check_target_files(Download download, __UNUSED const Object& object) {
+resume_check_target_files(Download download, [[maybe_unused]] const Object& object) {
   FileList* fileList = download.file_list();
 
   if (!fileList->is_open())
@@ -423,7 +393,7 @@ resume_check_target_files(Download download, __UNUSED const Object& object) {
   } else {
     // We consider empty file lists as being valid.
     return fileList->empty() || fileList->front()->is_created();
-  }    
+  }
 }
 
 void
@@ -433,19 +403,19 @@ resume_load_file_priorities(Download download, const Object& object) {
 
   const Object::list_type& files = object.get_key_list("files");
 
-  Object::list_const_iterator filesItr  = files.begin();
-  Object::list_const_iterator filesLast = files.end();
+  auto filesItr  = files.begin();
+  auto filesLast = files.end();
 
   FileList* fileList = download.file_list();
 
-  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
+  for (auto listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     if (filesItr == filesLast)
       return;
 
     // Update the priority from the fast resume data.
     if (filesItr->has_key_value("priority") &&
         filesItr->get_key_value("priority") >= 0 && filesItr->get_key_value("priority") <= PRIORITY_HIGH)
-      (*listItr)->set_priority((priority_t)filesItr->get_key_value("priority"));
+      (*listItr)->set_priority(static_cast<priority_enum>(filesItr->get_key_value("priority")));
 
     if (filesItr->has_key_value("completed"))
       (*listItr)->set_completed_chunks(filesItr->get_key_value("completed"));
@@ -454,18 +424,18 @@ resume_load_file_priorities(Download download, const Object& object) {
 
 void
 resume_save_file_priorities(Download download, Object& object) {
-  Object::list_type&    files    = object.insert_preserve_copy("files", Object::create_list()).first->second.as_list();
-  Object::list_iterator filesItr = files.begin();
+  auto& files    = object.insert_preserve_copy("files", Object::create_list()).first->second.as_list();
+  auto  filesItr = files.begin();
 
   FileList* fileList = download.file_list();
 
-  for (FileList::iterator listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
+  for (auto listItr = fileList->begin(), listLast = fileList->end(); listItr != listLast; ++listItr, ++filesItr) {
     if (filesItr == files.end())
       filesItr = files.insert(filesItr, Object::create_map());
     else if (!filesItr->is_map())
       *filesItr = Object::create_map();
 
-    filesItr->insert_key("priority", (int64_t)(*listItr)->priority());
+    filesItr->insert_key("priority", static_cast<int64_t>((*listItr)->priority()));
   }
 }
 
@@ -476,17 +446,15 @@ resume_load_addresses(Download download, const Object& object) {
 
   PeerList* peerList = download.peer_list();
 
-  const Object::list_type& src = object.get_key_list("peers");
-  
-  for (Object::list_const_iterator itr = src.begin(), last = src.end(); itr != last; ++itr) {
-    if (!itr->is_map() ||
-        !itr->has_key_string("inet") || itr->get_key_string("inet").size() != sizeof(SocketAddressCompact) ||
-        !itr->has_key_value("failed") ||
-        !itr->has_key_value("last") || itr->get_key_value("last") > cachedTime.seconds())
+  for (const auto& key : object.get_key_list("peers")) {
+    if (!key.is_map() ||
+        !key.has_key_string("inet") || key.get_key_string("inet").size() != sizeof(SocketAddressCompact) ||
+        !key.has_key_value("failed") ||
+        !key.has_key_value("last") || key.get_key_value("last") > this_thread::cached_seconds().count())
       continue;
 
     int flags = 0;
-    rak::socket_address socketAddress = *reinterpret_cast<const SocketAddressCompact*>(itr->get_key_string("inet").c_str());
+    rak::socket_address socketAddress = *reinterpret_cast<const SocketAddressCompact*>(key.get_key_string("inet").c_str());
 
     if (socketAddress.port() != 0)
       flags |= PeerList::address_available;
@@ -496,8 +464,8 @@ resume_load_addresses(Download download, const Object& object) {
     if (peerInfo == NULL)
       continue;
 
-    peerInfo->set_failed_counter(itr->get_key_value("failed"));
-    peerInfo->set_last_connection(itr->get_key_value("last"));
+    peerInfo->set_failed_counter(key.get_key_value("failed"));
+    peerInfo->set_last_connection(key.get_key_value("last"));
   }
 
   // Tell rTorrent to harvest addresses.
@@ -505,10 +473,9 @@ resume_load_addresses(Download download, const Object& object) {
 
 void
 resume_save_addresses(Download download, Object& object) {
-  const PeerList* peerList = download.peer_list();
-  Object&         dest     = object.insert_key("peers", Object::create_list());
+  auto& dest = object.insert_key("peers", Object::create_list());
 
-  for (PeerList::const_iterator itr = peerList->begin(), last = peerList->end(); itr != last; ++itr) {
+  for (const auto& dlp : *download.peer_list()) {
     // Add some checks, like see if there's anything interesting to
     // save, etc. Or if we can reconnect to it at some later time.
     //
@@ -518,13 +485,13 @@ resume_save_addresses(Download download, Object& object) {
 
     Object& peer = dest.insert_back(Object::create_map());
 
-    const rak::socket_address* sa = rak::socket_address::cast_from(itr->second->socket_address());
+    const rak::socket_address* sa = rak::socket_address::cast_from(dlp.second->socket_address());
 
     if (sa->family() == rak::socket_address::af_inet)
-      peer.insert_key("inet", std::string(SocketAddressCompact(sa->sa_inet()->address_n(), htons(itr->second->listen_port())).c_str(), sizeof(SocketAddressCompact)));
+      peer.insert_key("inet", std::string(SocketAddressCompact(sa->sa_inet()->address_n(), htons(dlp.second->listen_port())).c_str(), sizeof(SocketAddressCompact)));
 
-    peer.insert_key("failed",  itr->second->failed_counter());
-    peer.insert_key("last",    itr->second->is_connected() ? cachedTime.seconds() : itr->second->last_connection());
+    peer.insert_key("failed", dlp.second->failed_counter());
+    peer.insert_key("last", dlp.second->is_connected() ? this_thread::cached_seconds().count() : dlp.second->last_connection());
   }
 }
 
@@ -533,46 +500,46 @@ resume_load_tracker_settings(Download download, const Object& object) {
   if (!object.has_key_map("trackers"))
     return;
 
-  const Object& src = object.get_key("trackers");
-  TrackerList*  tracker_list = download.tracker_list();
+  auto& src          = object.get_key("trackers");
+  auto  tracker_list = download.main()->tracker_list();
 
-  for (Object::map_const_iterator itr = src.as_map().begin(), last = src.as_map().end(); itr != last; ++itr) {
-    if (!itr->second.has_key("extra_tracker") || itr->second.get_key_value("extra_tracker") == 0 ||
-        !itr->second.has_key("group"))
+  for (const auto& map : src.as_map()) {
+    if (!map.second.has_key("extra_tracker") || map.second.get_key_value("extra_tracker") == 0 ||
+        !map.second.has_key("group"))
       continue;
 
-    if (tracker_list->find_url(itr->first) != tracker_list->end())
+    if (tracker_list->find_url(map.first) != tracker_list->end())
       continue;
 
-    download.tracker_list()->insert_url(itr->second.get_key_value("group"), itr->first);
+    download.main()->tracker_list()->insert_url(map.second.get_key_value("group"), map.first);
   }
 
-  for (TrackerList::iterator itr = tracker_list->begin(), last = tracker_list->end(); itr != last; ++itr) {
-    if (!src.has_key_map((*itr)->url()))
+  for (auto tracker : *tracker_list) {
+    if (!src.has_key_map(tracker.url()))
       continue;
 
-    const Object& trackerObject = src.get_key((*itr)->url());
+    const Object& trackerObject = src.get_key(tracker.url());
 
     if (trackerObject.has_key_value("enabled") && trackerObject.get_key_value("enabled") == 0)
-      (*itr)->disable();
+      tracker.disable();
     else
-      (*itr)->enable();
-  }    
+      tracker.enable();
+  }
 }
 
 void
 resume_save_tracker_settings(Download download, Object& object) {
-  Object& dest = object.insert_preserve_copy("trackers", Object::create_map()).first->second;
-  TrackerList* tracker_list = download.tracker_list();
+  auto& dest         = object.insert_preserve_copy("trackers", Object::create_map()).first->second;
+  auto  tracker_list = download.main()->tracker_list();
 
-  for (TrackerList::iterator itr = tracker_list->begin(), last = tracker_list->end(); itr != last; ++itr) {
-    Object& trackerObject = dest.insert_key((*itr)->url(), Object::create_map());
+  for (auto tracker : *tracker_list) {
+    Object& trackerObject = dest.insert_key(tracker.url(), Object::create_map());
 
-    trackerObject.insert_key("enabled", Object((int64_t)(*itr)->is_enabled()));
+    trackerObject.insert_key("enabled", Object(static_cast<int64_t>(tracker.is_enabled())));
 
-    if ((*itr)->is_extra_tracker()) {
-      trackerObject.insert_key("extra_tracker", Object((int64_t)1));
-      trackerObject.insert_key("group", (*itr)->group());
+    if (tracker.is_extra_tracker()) {
+      trackerObject.insert_key("extra_tracker", Object(int64_t{1}));
+      trackerObject.insert_key("group", tracker.group());
     }
   }
 }
